#include <kvs/Vector3>
#include <kvs/AVSField>
#include <kvs/StructuredVolumeExporter>  

#include <kvs/PolygonObject>
#include <kvs/StructuredVolumeObject>
#include <kvs/Isosurface>
#include <kvs/glut/Application>
#include <kvs/glut/Screen>

inline float func( float x, float y, float z) {
  return ( 3.0 - x*x - y*y - 4.0*z*z );
}

int main( int argc, char** argv )
{
  // グリッドサイズ
  kvs::UInt32 gridNum = 64;
  const kvs::Vector3ui resol(gridNum, gridNum, gridNum);
  // ボリュームデータの値を格納するためのメモリの割り当て
  kvs::ValueArray<float> data( gridNum * gridNum * gridNum );
  
  // dataにを操作するためのポインタ変数
  float* pdata = data.data();  

  float min = -2.0, max = 2.0;
  float dt = (float)(max - min)/(float)(gridNum-1);

  kvs::UInt64 index = 0;
  for(int k=0; k< 64; k++) {
    for(int j = 0; j<64; j++) {
      for(int i=0; i<64; i++) {
	float x = (float)min + (float)i*dt; 	
	float y = (float)min + (float)j*dt; 
	float z = (float)min + (float)k*dt; 
	// 関数値を評価してdataへ格納
	pdata[ index++ ] =  func(x, y, z) ;
      }
    }
  }

  size_t veclen = 1.0;
  // 構造型ボリュームデータの生成
  kvs::StructuredVolumeObject *volume = new kvs::StructuredVolumeObject();
  volume->setGridType( kvs::StructuredVolumeObject::Uniform ); 
  volume->setVeclen( veclen ); 
  volume->setResolution( resol );
  volume->setValues( data );
  
  // ボリュームデータが最大値・最小値をもっているかチェック
  if ( !volume->hasMinMaxValues() ) 
    volume->updateMinMaxValues();     // もっていなければ計算する

  // 等値面の値をキーボードから入力する
  double input_iso, iso;
  double value_min = static_cast<double>( volume->minValue() );       // ボリュームデータの最小値を表示 
  double value_max = static_cast<double>( volume->maxValue() );       // ボリュームデータの最大値を表示
  std::cout << "Input Isolevel  [ " 
	    << value_min << " - " << value_max <<  "] : ";
  std::cin  >> input_iso;

  // 最大値と最小値を超えていないかチェック
  if     ( input_iso < value_min ) iso = value_min;
  else if( input_iso > value_max ) iso = value_max;
  else                             iso = input_iso;  

  kvs::PolygonObject* object = new kvs::Isosurface( volume,
						    iso,
						    kvs::PolygonObject::VertexNormal ); // Normal Type
  if ( !object )
    {
      kvsMessageError( "Cannot create a polygon object." );
      delete volume;
      return( false );
    }
  
  delete volume;
  
  // Screen.
  kvs::glut::Application app( argc, argv );
  
  kvs::glut::Screen screen( &app );
  screen.registerObject( object );
  screen.setGeometry( 0, 0, 512, 512 );
  screen.setTitle( "kvs::Isosurface" );
  screen.show();
  
  return( app.run() );    
}

